/** \file FileImport.h
    \brief Declaration of FileImport and related classes.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_FileImport_h
#define profit_FileImport_h

#include "RootFrame.h"

class TGTransientFrame;

#include <list>
#include <string>

#include "RQ_OBJECT.h"
#include "Rtypes.h"

namespace profit {

  class FileImport;
  class Gui;

  /** \class QuantityInterpreter
      \brief Class which creates Root primitive frame for interpreting a single data field as a physical quantity.
  */
  class QuantityInterpreter {
    public:
      /** \brief Create a QualityInterpreter frame inside the given composite frame.
          \param parent The parent frame in which to display this interpreter frame.
      */
      QuantityInterpreter(TGCompositeFrame * parent);

      virtual ~QuantityInterpreter();

    private:
      RootFrameTracker m_tracker;
  };

  /** \class ExtensionInterpreter
      \brief Class which creates Root primitive frame for interpreting an entire extension, which may include
             any number of quantity interpretation frames.
  */
  class ExtensionInterpreter {
    public:
      typedef std::list<QuantityInterpreter> cont_type;

      RQ_OBJECT("profit::ExtensionInterpreter")
      ClassDef(ExtensionInterpreter, 0)

      /** \brief Create ExtensionInterpreter frame inside the given composite frame.
          \param import The top level importer object on whose behalf this interpreter is being created.
          \param parent The parent frame in which to display this interpreter frame.
      */
      ExtensionInterpreter(FileImport * import, TGCompositeFrame * parent);

      virtual ~ExtensionInterpreter();

      /// \brief Add a new QuantityInterpreter, in response to clicking the Add Quantity button.
      virtual void handleAddInterpreter();

    private:
      RootFrameTracker m_tracker;
      cont_type m_interpreter;
      FileImport * m_import;
      TGCompositeFrame * m_interp_frame;
  };

  /** \class FileInterpreter
      \brief Class which creates Root primitive frame for interpreting an entire file, which may include
             any number of QuantityInterpeter frames and/or ExtensionInterpreter frames.
  */
  class FileInterpreter {
    public:
      typedef std::list<ExtensionInterpreter *> cont_type;

      RQ_OBJECT("profit::FileInterpreter")
      ClassDef(FileInterpreter, 0)

      /** \brief Create FileInterpreter Root frame inside the given composite frame.
          \param import The top level importer object on whose behalf this interpreter is being created.
          \param parent The parent Root frame in which to display this interpreter frame.
      */
      FileInterpreter(FileImport * import, TGCompositeFrame * parent);

      virtual ~FileInterpreter();

      /// \brief Add a new ExtensionInterpreter, in response to clicking the Add Extension button.
      virtual void handleAddInterpreter();

    private:
      RootFrameTracker m_tracker;
      cont_type m_interpreter;
      FileImport * m_import;
      TGCompositeFrame * m_interp_frame;
  };

  /** \class FileImport
      \brief Class which creates Root primitive frame for interpreting any number of data files.
  */
  class FileImport {
    public:
      typedef std::list<FileInterpreter *> cont_type;

      RQ_OBJECT("profit::FileImport")
      ClassDef(FileImport, 0)

      /** \brief Create FileImport dialog box associated with the given Gui object.
          \param gui The top-level gui.
      */
      FileImport(Gui * gui);

      virtual ~FileImport();

      /** \brief For testing purposes only, run in test mode, which causes the dialog box
          to be displayed, then various methods exercised, rather than just displaying the dialog box.
          TODO: Note this doesn't really work correctly.
      */
      virtual void runInTestMode();

      virtual void handleCloseWindow();

      /// \brief Add a new FileInterpreter, in response to clicking the Add File button.
      virtual void handleAddInterpreter();

      /// \brief Make all necessary changes to layout of the dialog box.
      virtual void updateDisplay();

    private:
      TGTransientFrame * buildGui();

      RootFrameTracker m_tracker;
      cont_type m_interpreter;
      Gui * m_gui;
      TGTransientFrame * m_transient_frame;
      TGCompositeFrame * m_interp_frame;
  };

}

#endif
