/** \file Tracker.h
    \brief Declaration of Tracker class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_Tracker_h
#define profit_Tracker_h

#include <map>
#include <string>
#include <utility>

namespace profit {

  /** \class Tracker
      \brief Class which acts as a single point of access for tracking
             a group of objects, allowing them to be looked up based on
             a string key.
  */
  template <typename T>
  class Tracker {
    public:
      typedef std::multimap<std::string, T> cont_type;
      typedef typename cont_type::iterator iterator;
      typedef typename cont_type::const_iterator const_iterator;
      typedef std::pair<iterator, iterator> range_iterator;
      typedef std::pair<const_iterator, const_iterator> const_range_iterator;

      void insert(const std::string & id, const T & object);

      void remove(const std::string & id, const T & object);

      const_iterator find(const std::string & id) const;

      const_range_iterator equal_range(const std::string & id) const;

      const_iterator begin() const;

      const_iterator end() const;

    protected:
      iterator find(const std::string & id, const T & object);

      cont_type m_cont;
  };

  template <typename T>
  inline void Tracker<T>::insert(const std::string & id, const T & object) {
    // Look for the object.
    iterator itor = find(id, object);

    // If this object is not already tracked, track it.
    if (m_cont.end() == itor) m_cont.insert(typename cont_type::value_type(id, object));
  }

  template <typename T>
  inline void Tracker<T>::remove(const std::string & id, const T & object) {
    // Look for the object.
    iterator itor = find(id, object);

    // If this object is tracked, un-track it.
    if (m_cont.end() != itor) m_cont.erase(itor);
  }

  template <typename T>
  inline typename Tracker<T>::const_iterator Tracker<T>::find(const std::string & id) const { return m_cont.find(id); }

  template <typename T>
  inline typename Tracker<T>::const_range_iterator Tracker<T>::equal_range(const std::string & id) const {
    return m_cont.equal_range(id);
  }

  template <typename T>
  inline typename Tracker<T>::const_iterator Tracker<T>::begin() const { return m_cont.begin(); }

  template <typename T>
  inline typename Tracker<T>::const_iterator Tracker<T>::end() const { return m_cont.end(); }

  template <typename T>
  inline typename Tracker<T>::iterator Tracker<T>::find(const std::string & id, const T & object) {
    // Find all objects which have this identifier.
    range_iterator range = m_cont.equal_range(id);

    // Look for an exact match to the current object being tracked.
    iterator itor = range.first;
    for (; itor != range.second && object != itor->second; ++itor) {}

    // If the object was not found in the range of equal objects, set iterator to point to the end of container.
    if (itor == range.second) itor = m_cont.end();

    return itor;
  }

}

#endif
